"""Create mesh for lwru2 validation of the Oshiri island tsunami
"""

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# Assume that the root AnuGA dir is included in your pythonpath
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

from pmesh.mesh import *
from pyvolution.coordinate_transforms.geo_reference import Geo_reference

#-------------------------------------------------------------
if __name__ == "__main__":


    #Basic geometry
    
    xleft   = 0
    xright  = 5.448
    ybottom = 0
    ytop    = 3.402

    #Outline 
    point_sw = [xleft, ybottom]
    point_se = [xright, ybottom]
    point_nw = [xleft, ytop]    
    point_ne = [xright, ytop]

    #Midway points (left)
    point_mtop = [xleft + (xright-xleft)/3+1, ytop]
    point_mbottom = [xleft + (xright-xleft)/3+1, ybottom]


    geo = Geo_reference(xllcorner = xleft,
                        yllcorner = ybottom)
    
                        
    print "***********************"
    print "geo ref", geo
    print "***********************"
    
    m = Mesh(geo_reference=geo)

    #Boundary
    dict = {}
    dict['points'] = [point_se,   #se
                      point_ne,
                      point_mtop,
                      point_nw,
                      point_sw,
                      point_mbottom]

    
    dict['segments'] = [[0,1], [1,2], [2,3], [3,4],
                        [4,5], [5,0],  #The outer border
                        [2,5]]         #Separator
    
    dict['segment_tags'] = ['wall',
                            'wall',
                            'wall',
                            'wave',
                            'wall',
                            'wall',
                            '']           #Interior

        
    m.addVertsSegs(dict)





    #Localised refined area for gulleys
    dict = {}
    xl = 4.8
    xr = 5.3
    yb = 1.6
    yt = 2.3
    p0 = [xl, yb]
    p1 = [xr, yb]
    p2 = [xr, yt]
    p3 = [xl, yt]
    
    dict['points'] = [p0, p1, p2, p3]
    dict['segments'] = [[0,1], [1,2], [2,3], [3,0]]
    dict['segment_tags'] = ['', '', '', '']
    m.addVertsSegs(dict)    

    #Island area
    island_0 = [xleft + 2*(xright-xleft)/3+1.2, ytop-0.5]
    island_1 = [xleft + 2*(xright-xleft)/3+0.5, ybottom + 2*(ytop-ybottom)/3]
    island_2 = [xleft + (xright-xleft)/2+0.3, ybottom + 2*(ytop-ybottom)/3-0.3]
    island_3 = [xleft + (xright-xleft)/2+0.3, ybottom + (ytop-ybottom)/3+0.3]
    island_4 = [xleft + 2*(xright-xleft)/3+0.5, ybottom + (ytop-ybottom)/3] 
    island_5 = [xleft + 2*(xright-xleft)/3+1.2, ybottom+0.2]
    island_6 = [xl-.01, yb]  #OK
    island_7 = [xl-.01, yt]  #OK      
    

    dict['points'] = [island_0, island_1, island_2,
                      island_3, island_4, island_5,
                      p0, p3]
                      #island_6, island_7]

    #Both versions fail
    #dict['segments'] = [[0,1], [1,2], [2,3], [3,4],
    #                    [4,5], [5,6], [7,0]]
    #dict['segment_tags'] = ['', '', '', '', '', '', '']    

    dict['segments'] = [[0,1], [1,2], [2,3], [3,4],
                        [4,5], [5,6], [6,7], [7,0]]    
    dict['segment_tags'] = ['', '', '', '', '', '', '', '']

    #ALso try having list of segment_tags the wrong length


    m.addVertsSegs(dict)    
    
#
    
    base_resolution = 1000

    ocean = m.addRegionEN(xleft+.1, ybottom+.1)
    ocean.setMaxArea(0.1*base_resolution)

    mid = m.addRegionEN(point_mbottom[0]+.1, ybottom+.1)
    mid.setMaxArea(0.05*base_resolution)
    

    #inner = m.addRegionEN(island_3[0]+.1, island_3[1]+.1)
    #inner.setMaxArea(0.0005*base_resolution)


    gulleys = m.addRegionEN(p0[0]+0.1, p0[1]+0.1)
    gulleys.setMaxArea(0.00002*base_resolution)    
    
    
    m.generateMesh('pzq28.0za1000000a')

    import filenames
    m.export_mesh_file(filenames.mesh_filename)
    
